@@ -18,7 +18,9 @@ from polly.constants import OBSOLETE_METADATA_FIELDS, COHORT_VERSION, dot
 import shutil
 from joblib import Parallel, delayed
 from cmapPy.pandasGEXpress.parse import parse
+from cmapPy.pandasGEXpress.write_gct import write
 from cmapPy.pandasGEXpress.concat import assemble_data
+from cmapPy.pandasGEXpress import GCToo
 
 
 class CohortFileStandard:
@@ -606,3 +608,114 @@ class CohortFileStandard:
         logging.basicConfig(level=logging.INFO)
         logging.info("Cohort Deleted Successfuly!")
         self._cohort_details = None
+
+    def _rename_samples(self, dataset_id: str, gct_object: GCToo) -> GCToo:
+        """
+        Adds dataset_id as prefix to the sample names (cids) in the gct object
+        """
+        gct_object.data_df.columns = [
+            dataset_id + "_" + sample for sample in gct_object.data_df.columns
+        ]
+        gct_object.col_metadata_df.index = [
+            [dataset_id + "_" + sample for sample in gct_object.col_metadata_df.index]
+        ]
+        return gct_object
+
+    def merge_gcts(
+        self, entity_type: str, cohort_path: str, file_name: str, file_path: str
+    ) -> None:
+        """
+        Merge the input gcts together to create one gct object. Only the features(genes) common to all the gcts are retained.
+        """
+        dataset_ids = list(self._cohort_details["entity_id"].keys())
+        omixatlas_details = self._cohort_details.get("source_omixatlas")
+        gct_files = [
+            f"{cohort_path}/{self._cohort_details.get('entity_id',{}).get(dataset_id)[0]}_{dataset_id}.gct"
+            for dataset_id in dataset_ids
+        ]
+        gct_objects = {
+            dataset_id: parse(gct_file)
+            for dataset_id, gct_file in zip(dataset_ids, gct_files)
+        }
+        # Creating a dict for dataset_id to samples mapping with dataset_id prefixed
+        dataset_sample_mapping = {}
+        # Collecting samples by differentiating between entity_types of cohort
+        if entity_type == "sample":
+            # Cohort contains repositories with multiple mapped datasets
+            for dataset in dataset_ids:
+                omixatlas = self._cohort_details.get("entity_id", {}).get(dataset, {})[
+                    0
+                ]
+                samples = omixatlas_details.get(omixatlas, {}).get(dataset)
+                dataset_sample_mapping[dataset] = [
+                    f"{dataset}_{sample}" for sample in samples
+                ]
+        else:
+            # Cohort contains repositories with single mapped datasets
+            for dataset in dataset_ids:
+                samples = list(gct_objects[dataset].col_metadata_df.index)
+                dataset_sample_mapping[dataset] = [
+                    f"{dataset}_{sample}" for sample in samples
+                ]
+        # Dictionaries for parsed data
+        data_df_dict = {}
+        col_metadata_dict = {}
+        for dataset_id in dataset_ids:
+            # Rename all samples for each gct and add dataset_id as a column to col metadata of each gct object
+            gct_objects[dataset_id] = self._rename_samples(
+                dataset_id, gct_objects[dataset_id]
+            )
+            gct_objects[dataset_id].col_metadata_df["dataset_id"] = dataset_id
+            # Separating data_df and col_metadata_df, with tailored index and columns according to samples
+            samples = dataset_sample_mapping[dataset_id]
+            sampled_data_df = gct_objects[dataset_id].data_df[samples]
+            sampled_col_metadata_df = gct_objects[dataset_id].col_metadata_df.loc[
+                samples
+            ]
+            # Step to ensure index is a proper list
+            sampled_col_metadata_df.index = samples
+            data_df_dict[dataset_id] = sampled_data_df
+            col_metadata_dict[dataset_id] = sampled_col_metadata_df
+
+        if len(dataset_ids) == 1:
+            # No need to merge as only one dataset present
+            merged_data_df = data_df_dict[dataset_ids[0]]
+            merged_col_metadata_df = col_metadata_dict[dataset_ids[0]]
+            merged_row_metadata_df = gct_objects[dataset_ids[0]].row_metadata_df.loc[
+                merged_data_df.index
+            ]
+        else:
+            merged_data_df = pd.merge(
+                data_df_dict[dataset_ids[0]],
+                data_df_dict[dataset_ids[1]],
+                left_index=True,
+                right_index=True,
+            )
+            merged_col_metadata_df = pd.concat(
+                [
+                    col_metadata_dict[dataset_ids[0]],
+                    col_metadata_dict[dataset_ids[1]],
+                ],
+                axis=0,
+            )
+            if len(dataset_ids) > 2:
+                for dataset_id in dataset_ids[2:]:
+                    merged_data_df = pd.merge(
+                        merged_data_df,
+                        data_df_dict[dataset_id],
+                        left_index=True,
+                        right_index=True,
+                    )
+                    merged_col_metadata_df = pd.concat(
+                        [merged_col_metadata_df, col_metadata_dict[dataset_id]],
+                        axis=0,
+                    )
+            merged_row_metadata_df = gct_objects[dataset_ids[0]].row_metadata_df.loc[
+                merged_data_df.index
+            ]
+        merged_gct_object = GCToo.GCToo(
+            data_df=merged_data_df,
+            row_metadata_df=merged_row_metadata_df,
+            col_metadata_df=merged_col_metadata_df,
+        )
+        write(merged_gct_object, helpers.make_path(file_path, file_name))
