@@ -75,6 +75,36 @@ def parameter_check_for_dataset_id(dataset_id):
         )
 
 
+def parameter_check_for_list_dataset_ids(dataset_ids):
+    """Checking for validity of repo id
+    Args:
+        dataset_id (): Dataset Id of the dataset
+
+    Raises:
+        paramException: Error if dataset id is empty or is not str
+    """
+    if not (dataset_ids and isinstance(dataset_ids, list)):
+        raise paramException(
+            title="Param Error",
+            detail="dataset_ids should be list of strings",
+        )
+
+
+def str_params_check(str_params: list):
+    """Checking if string parameters are of valid format
+    Args:
+        str_params (list): list of string parameters
+
+    Raises:
+        paramException: Error if any of string parameters are empty or is not str
+    """
+    for param in str_params:
+        if not isinstance(param, str):
+            raise paramException(
+                title="Param Error", detail=f"{param} should be a string"
+            )
+
+
 def make_repo_id_string(repo_id: int) -> str:
     """If repo id is int, change to string
 
@@ -758,11 +788,7 @@ def move_data_params_check(
         parameter_check_for_repo_id(src_repo_key)
         parameter_check_for_repo_id(dest_repo_key)
         parameter_check_for_priority(priority)
-        if not dataset_ids or not isinstance(dataset_ids, list):
-            raise paramException(
-                title="param error",
-                detail="dataset_ids should not be empty and dataset_ids should be in list format.",
-            )
+        parameter_check_for_list_dataset_ids(dataset_ids)
     except Exception as err:
         raise err
 
@@ -951,11 +977,7 @@ def check_create_omixatlas_parameters(
     """
     try:
         str_params = [display_name, description, repo_name, image_url]
-        for param in str_params:
-            if not isinstance(param, str):
-                raise paramException(
-                    title="Param Error", detail=f"{param} should be a string"
-                )
+        str_params_check(str_params)
         check_components_parameter(components)
 
         if not isinstance(category, str) or (
@@ -1054,11 +1076,7 @@ def check_update_omixatlas_parameters(
             )
 
         str_params = [display_name, description, image_url, workspace_id]
-        for param in str_params:
-            if not isinstance(param, str):
-                raise paramException(
-                    title="Param Error", detail=f"{param} should be a string"
-                )
+        str_params_check(str_params)
         check_components_parameter(components)
     except Exception as err:
         raise err
@@ -1094,6 +1112,11 @@ def compare_repo_id_and_id(repo_id: str, id: str):
         )
 
 
+# TODO -> can we remove this function altogether
+# and use extract_error_message_details in errors module
+# to follow the DRY principle
+# extract_error_message should be at only 1 place and should be generic enough
+# to handle all the cases
 def extract_error_message(error_msg) -> str:
     """
     Extract error message from the error
@@ -1105,20 +1128,12 @@ def extract_error_message(error_msg) -> str:
     if "detail" in error:
         detail = error.get("detail")
 
-    return detail
+    if "title" in error:
+        title = error.get("title")
 
+    error_msg_dict = {"title": title, "detail": detail}
 
-def dataset_id_type_check_in_delete_datasets(dataset_ids: list):
-    """Dataset Id type check in delete datasets
-
-    Args:
-        dataset_id (list): dataset ids list
-    """
-    if not (dataset_ids and isinstance(dataset_ids, list)):
-        raise paramException(
-            title="Param Error",
-            detail="dataset_ids should be list of strings",
-        )
+    return error_msg_dict
 
 
 def dataset_file_path_dict_type_check_in_delete_datasets(dataset_file_path_dict: dict):
@@ -1162,7 +1177,7 @@ def parameter_check_for_delete_dataset(
     """
     try:
         parameter_check_for_repo_id(repo_id)
-        dataset_id_type_check_in_delete_datasets(dataset_ids)
+        parameter_check_for_list_dataset_ids(dataset_ids)
         dataset_file_path_dict_type_check_in_delete_datasets(dataset_file_path_dict)
     except Exception as err:
         raise err
@@ -1219,55 +1234,188 @@ def dataset_file_path_is_subset_dataset_id(
         print("\n")
 
 
+def fetch_list_api_control_levers() -> str:
+    """Fetch List Files API control Levers"""
+    response = requests.get(const.LIST_FILE_API_CONTROL_LEVER_LINK)
+    error_handler(response)
+    control_levers_dict = json.loads(response.text)
+    default_page_size = control_levers_dict.get(
+        const.DEFAULT_PAGE_SIZE_LIST_FILES_KEY_NAME
+    )
+    page_reduction_percentage = control_levers_dict.get(
+        const.PAGE_REDUCTION_PERCENTAGE_LIST_FILES_KEY_NAME
+    )
+    return default_page_size, page_reduction_percentage
+
+
+def reduce_page_size(default_page_size: int, reduction_percentage: int) -> int:
+    """Reduce the page size based on the current reduction percentage
+
+    Args:
+        default_page_size (int): current page size
+
+    Returns:
+        int: reduced page size
+    """
+    # error_msg_dict = extract_error_message(response.text)
+    # reduce the default page size
+
+    # reduce page size by PAGE_REDUCTION_PERCENTAGE_LIST_FILES
+    # reduction_multiplier = (100 - const.PAGE_REDUCTION_PERCENTAGE_LIST_FILES) / 100
+
+    reduction_multiplier = (100 - reduction_percentage) / 100
+
+    default_page_size = reduction_multiplier * default_page_size
+
+    # TODO -> Put tracking to see how many times API is crashing
+    # with the current default page size -> the tracking will help
+    # to optimise the control lever
+
+    # give nearest rounded integer value -> so that decimal does not come
+    return round(default_page_size)
+
+
+def extract_page_after_from_next_link(next_link: str) -> int:
+    """Extract page[after] from next_link
+
+    Args:
+        next_link (str): next_link given by the API in pagination
+
+    Returns:
+        int: page_after
+    """
+    # next link format ->
+    # /repositories/1643016586529/files
+    # ?page[size]=2500&page[after]=2500&version=latest&include_metadata=True
+
+    # if page[after] exists in next_link then the page[after] value will be b/w
+    # `page[after]=` and `&`
+
+    if "page[after]" in next_link:
+        page_after_str = helpers.find_between(next_link, "page[after]=", "&")
+        # typecast the page[after] value to int and return
+        return int(page_after_str)
+    else:
+        # return 0 as default page_size if "page[after]" does not exist
+        # in the next_link -> this is an extreme case where the API is broken
+        # in normal cases this error may never come
+        return 0
+
+
 # cached for the cases when this function is called internally when same
 # result is needed multiple times
 @lru_cache(maxsize=None)
 def list_files(
-    self: dict, repo_id: str, metadata="true", data="true", version="current"
+    self, repo_id: str, metadata="true", data="true", version="current"
 ) -> list:
     """helper function to integrate list files API response
 
     Args:
-        self (dict): dictionary storing session vara
+        self (polly_session_object): polly_session
         repo_id (str): repo id of the omixatlas
     Returns:
         list_files_resp -> list of objects with requests type
     """
-    # page_size -> for paginating the API
+    # endpoints for list files API
     files_api_endpoint = f"{self.discover_url}/repositories/{repo_id}/files"
-    # initialising an empty value of next link
+
+    # initialising an empty string of next link
     next_link = ""
     responses_list = []
-    # once the pages will end -> next_link will be
-    # set to None by the API
+
+    # page_size -> for paginating the API
+    # set to the default page size mentioned in the constants
+    # if the API crashes then -> it will reduced
+    # default_page_size = const.DEFAULT_PAGE_SIZE_LIST_FILES
+    default_page_size, reduction_percentage = fetch_list_api_control_levers()
+
+    # initially set to 0, but will be updated
+    # as the API request crashes request is called on next link
+    # in that next_link will be set to empty string and page_after
+    # will be updated to current page_after in the next_link
+    page_after = 0
+
+    # initially next_link will be empty string
+    # once the pages will end -> next_link will be set to None by the API
     while next_link is not None:
         if next_link:
             next_endpoint = f"{self.discover_url}{next_link}"
             response = self.session.get(next_endpoint)
+
+            if response.status_code == http_codes.PAYLOAD_TOO_LARGE:
+                page_after = extract_page_after_from_next_link(next_link)
+                list_api_crash_messaging_tracking(self, page_after, default_page_size)
+
+                default_page_size = reduce_page_size(
+                    default_page_size, reduction_percentage
+                )
+                helpers.debug_print(
+                    self, f"--reduced page size---: {default_page_size}"
+                )
+                # intialise next_link to empty str so that query_params
+                # dict is intialised again with new default_page_size and page_after
+                next_link = ""
+                # go to the next iteation
+                continue
         else:
-            # getting the response for the first 100 pages initially
-            # 100 page is set for now -> so that data does not max out
-            # TODO -> need to be optimised for page size and time
             query_params = {
-                "page[size]": 100,
-                "page[after]": 0,
+                "page[size]": default_page_size,
+                "page[after]": page_after,
                 "include_metadata": f"{metadata}",
                 "data": f"{data}",
                 "version": f"{version}",
             }
+
             response = self.session.get(files_api_endpoint, params=query_params)
-        response.raise_for_status()
+            # in case of payload too large error, reduce the page size
+            if response.status_code == http_codes.PAYLOAD_TOO_LARGE:
+                list_api_crash_messaging_tracking(self, page_after, default_page_size)
+
+                default_page_size = reduce_page_size(
+                    default_page_size, reduction_percentage
+                )
+
+                helpers.debug_print(
+                    self, f"--reduced page size---: {default_page_size}"
+                )
+                # go to the next iteation
+                continue
+
+        error_handler(response)
         response_json = response.json()
         # list of request objects
-        # request object having both status and data of the response
+        # reponse object having both status and data of the response
         responses_list.append(response)
         # seeing after 1000 pages whose response is already fetched
         # if there are more pages
         next_link = response_json.get("links").get("next")
-
+        helpers.debug_print(self, f"next link--: {next_link}")
     return responses_list
 
 
+def list_api_crash_messaging_tracking(self, page_after: int, default_page_size: int):
+    """Function to print API crashing and log events for tracking
+
+    Args:
+        page_after (int): page_after value after which API crashed
+        default_page_size (int): page_size for which API crashed
+    """
+    helpers.debug_print(self, "------API crashed-------")
+    helpers.debug_print(self, f"----current page[after] value: {page_after}")
+
+    helpers.debug_print(self, f"--current_page_size: {default_page_size}")
+
+    # tracking metadata
+    # what parameters need to be shown on tracking dashboard
+    # put in the properties dict and pass it
+    properties_dict = {}
+    properties_dict["api_name"] = "list_file_api"
+    properties_dict["crashed_page_size"] = default_page_size
+    properties_dict["current_page_after"] = page_after
+
+    helpers.debug_logger(self, properties_dict)
+
+
 def normalize_file_paths(dataset_file_path_dict: dict):
     """Normalise all the file paths in the dataset_file_path_dict
 
@@ -1442,11 +1590,7 @@ def get_all_file_paths_param_check(repo_id: int, dataset_id: str):
     """
     try:
         parameter_check_for_repo_id(repo_id)
-        if not (dataset_id and isinstance(dataset_id, str)):
-            raise paramException(
-                title="Param Error",
-                detail="dataset_ids should be a string",
-            )
+        parameter_check_for_dataset_id(dataset_id)
     except Exception as err:
         raise err
 
@@ -1628,16 +1772,12 @@ It should either be a string or an integer. Please try again.",
 def param_check_download_dataset(
     repo_key: str, dataset_ids: list, folder_path: str
 ) -> None:
+    parameter_check_for_list_dataset_ids(dataset_ids)
     if not (repo_key and isinstance(repo_key, str)):
         raise paramException(
             title="Param Error",
             detail="repo_key (either id or name) is required and should be a string",
         )
-    if not (dataset_ids and isinstance(dataset_ids, list)):
-        raise paramException(
-            title="Param Error",
-            detail="dataset_ids is required and should be a list of dataset_ids as strings",
-        )
     if (not isinstance(folder_path, str)) or (not os.path.isdir(folder_path)):
         raise paramException(
             title="Param Error",
@@ -1728,3 +1868,38 @@ It should either be a string or an integer. Please try again.",
             title="Param Error",
             detail="Argument 'dataset_id' is either empty or invalid. It should be a string. Please try again.",
         )
+
+
+def return_sorted_dict(single_dict: dict) -> dict:
+    """
+    Function that takes in a dictionary and returns an alphabetically sorted dictionary,
+    except for keys - dataset_id and src_dataset_id.
+    If either/both of the two keys are present,it will get inserted in the beginning of the dictionary.
+    """
+    # checking presence of either of the dataset_id related cols in the df
+    id_cols_present = set.intersection(
+        set(["dataset_id", "src_dataset_id"]), set(single_dict.keys())
+    )
+    final_dict = {}
+    if len(id_cols_present) == 0:
+        # None of the two keys present, sort the dictionary
+        final_dict = dict(sorted(single_dict.items()))
+    elif len(id_cols_present) == 1:
+        # Sort the dictionary except for the key that is present,
+        # insert the key after the rest of the dict is sorted
+        key = id_cols_present.pop()
+        col_data = single_dict.pop(key)
+        sorted_dict = dict(sorted(single_dict.items()))
+        final_dict[key] = col_data
+        # updating the dictionary to append the dataset keys first
+        final_dict.update(sorted_dict)
+    else:
+        # Both the keys present, sort the rest of the dict and insert the keys at the beginning of the sorted dictionary.
+        dataset_id_data = single_dict.pop("dataset_id")
+        src_dataset_id_data = single_dict.pop("src_dataset_id")
+        sorted_dict = dict(sorted(single_dict.items()))
+        final_dict["dataset_id"] = dataset_id_data
+        final_dict["src_dataset_id"] = src_dataset_id_data
+        # updating the dictionary to append the dataset keys first
+        final_dict.update(sorted_dict)
+    return final_dict
