@@ -2,7 +2,6 @@ from polly import omixatlas
 import polly
 from polly.auth import Polly
 import os
-import csv
 from polly.errors import (
     RequestException,
     apiErrorException,
@@ -32,8 +31,10 @@ from polly_validator.validators import dataset_metadata_validator
 
 key = "POLLY_REFRESH_TOKEN"
 token = os.getenv(key)
+
 test_key = "TEST_POLLY_REFRESH_TOKEN"
 testpolly_token = os.getenv(test_key)
+
 dev_key = "DEV_POLLY_REFRESH_TOKEN"
 devpolly_token = os.getenv(dev_key)
 
@@ -540,34 +541,157 @@ def test_delete_empty_repo_id():
         omix_obj.delete_datasets(repo_id, dataset_ids)
 
 
-def test_query_metadata():
+@pytest.fixture()
+def get_query_metadata_fixture():
+    """Get success response for query_metadata enpoint from github"""
+    base_polly_py_test_url = test_const.POLLY_PY_TEST_FILES_URL
+
+    data_file = f"{base_polly_py_test_url}/query_metadata_success_response.json"
+    data = requests.get(data_file)
+    error_handler(data)
+    return data
+
+
+@pytest.fixture()
+def get_download_url_fixture():
+    """Get success response of dummy urls from github"""
+    base_polly_py_test_url = test_const.POLLY_PY_TEST_FILES_URL
+
+    data_file = f"{base_polly_py_test_url}/download_urls_response.json"
+    data = requests.get(data_file)
+    error_handler(data)
+    return data
+
+
+def test_query_metadata(mocker):
+    """
+    Mock function for query_metadata().
+    """
     Polly.auth(token)
-    nobj3 = omixatlas.OmixAtlas()
-    obj3 = omixatlas.OmixAtlas(token)
-    query_dict = {}
-    with open("tests/query.csv") as csv_file:
-        csv_reader = csv.reader(csv_file, delimiter=",")
-        line_count = 0
-        for row in csv_reader:
-            if line_count == 0:
-                line_count += 1
-            else:
-                query_dict[row[0]] = row[1]
-    query_dataset_level = query_dict["query_dataset_level"]
-    query_sample_level = query_dict["query_sample_level"]
-    query_feature_level = query_dict["query_feature_level"]
-    query_singlecell_sample = query_dict["query_singlecell_sample"]
-    query_singlecell_feature = query_dict["query_singlecell_feature"]
-    assert dict(obj3.query_metadata(query_feature_level)) is not None
-    assert dict(obj3.query_metadata(query_sample_level)) is not None
-    assert dict(obj3.query_metadata(query_dataset_level)) is not None
-    assert dict(obj3.query_metadata(query_singlecell_sample)) is not None
-    assert dict(obj3.query_metadata(query_singlecell_feature)) is not None
-    assert dict(nobj3.query_metadata(query_feature_level)) is not None
-    assert dict(nobj3.query_metadata(query_sample_level)) is not None
-    assert dict(nobj3.query_metadata(query_dataset_level)) is not None
-    assert dict(nobj3.query_metadata(query_singlecell_sample)) is not None
-    assert dict(nobj3.query_metadata(query_singlecell_feature)) is not None
+    obj = omixatlas.OmixAtlas()
+    query = test_const.SAMPLE_QUERY
+    mocker.patch(
+        polly.omixatlas.__name__ + ".OmixAtlas._get_query_id",
+        return_value="query_id",
+    )
+    mocker.patch(
+        polly.omixatlas.__name__ + ".OmixAtlas._process_query_to_completion",
+        return_value=pd.DataFrame(),
+    )
+    assert type(obj.query_metadata(query)) is pd.DataFrame
+
+
+def test_query_metadata_iterator(mocker, generator_function):
+    """
+    Mock function for query_metadata_iterator().
+    """
+    Polly.auth(token)
+    query = test_const.SAMPLE_QUERY
+    mocker.patch(
+        polly.omixatlas.__name__ + ".OmixAtlas._get_query_id",
+        return_value="query_id",
+    )
+    mocker.patch(
+        polly.omixatlas.__name__ + ".OmixAtlas._process_query_to_completion",
+        return_value=generator_function,
+    )
+    obj = omixatlas.OmixAtlas(token)
+    gen = obj.query_metadata(query)
+    assert gen is not None
+
+
+@pytest.fixture()
+def generator_function():
+    """
+    A dummy generator function for mocking a generator
+    """
+    dummy_list = [{"key": "value"}]
+    for i in dummy_list:
+        yield i
+
+
+def test_process_query_to_completion_iterator(
+    mocker, generator_function, get_query_metadata_fixture
+):
+    """
+    Mock function for _process_query_to_completion() with iterator.
+    """
+    Polly.auth(token)
+    obj = omixatlas.OmixAtlas()
+    mocked_response = get_query_metadata_fixture
+    response = mocker.patch.object(obj.session, "get")
+    response.return_value.status_code = 200
+    response.return_value = mocked_response
+    mocker.patch(
+        polly.omixatlas.__name__ + ".OmixAtlas._fetch_results_as_file",
+        return_value=generator_function,
+    )
+    query_id = "query_id"
+    iterator_function = True
+    gen = obj._process_query_to_completion(query_id, iterator_function)
+    assert gen is not None
+
+
+def test_process_query_to_completion(mocker, get_query_metadata_fixture):
+    """
+    Mock function for _process_query_to_completion() without iterator.
+    """
+    Polly.auth(token)
+    obj = omixatlas.OmixAtlas()
+    mocked_response = get_query_metadata_fixture
+    response = mocker.patch.object(obj.session, "get")
+    response.return_value.status_code = 200
+    response.return_value = mocked_response
+    mocker.patch(
+        polly.omixatlas.__name__ + ".OmixAtlas._fetch_results_as_file",
+        return_value=pd.DataFrame(),
+    )
+    query_id = "query_id"
+    iterator_function = False
+    df = obj._process_query_to_completion(query_id, iterator_function)
+    assert type(df) is pd.DataFrame
+
+
+def test_fetch_results_as_file(mocker, get_download_url_fixture):
+    """
+    Mock function for _fetch_results_as_file() without iterator.
+    """
+    Polly.auth(token)
+    obj = omixatlas.OmixAtlas()
+    mocked_response = get_download_url_fixture
+    response = mocker.patch.object(obj.session, "get")
+    response.return_value.status_code = 200
+    response.return_value = mocked_response
+    mocker.patch(
+        polly.omixatlas.__name__ + ".OmixAtlas._extract_results_from_download_urls",
+        return_value=pd.DataFrame(),
+    )
+    query_id = "query_id"
+    iterator_function = False
+    df = obj._fetch_results_as_file(query_id, iterator_function)
+    assert type(df) is pd.DataFrame
+
+
+def test_fetch_results_as_file_iterator(
+    mocker, generator_function, get_download_url_fixture
+):
+    """
+    Mock function for _fetch_results_as_file() with iterator.
+    """
+    Polly.auth(token)
+    obj = omixatlas.OmixAtlas()
+    mocked_response = get_download_url_fixture
+    response = mocker.patch.object(obj.session, "get")
+    response.return_value.status_code = 200
+    response.return_value = mocked_response
+    mocker.patch(
+        polly.omixatlas.__name__ + ".OmixAtlas._extract_results_from_download_urls",
+        return_value=generator_function,
+    )
+    query_id = "query_id"
+    iterator_function = True
+    gen = obj._fetch_results_as_file(query_id, iterator_function)
+    assert gen is not None
 
 
 def test_generate_report():
@@ -595,22 +719,93 @@ def test_generate_report():
         obj.generate_report(valid_repo_key, valid_dataset_id, invalid_workspace_id)
 
 
-'''def test_update_function_by_updating_description():
-    # Either value will be updated to the current value
-    # Or it will be the current value
-    # If value is not equal to the current value means
-    # update is not working
+# update omixatlas tests
 
-    description = "updated description ingestion_test_1 omixatlas"
-    # ingestion_test_1
-    repo_key = "1654268055800"
+
+# Test 1 -> pass no parameters to update -> will raise an error
+def test_update_oa_no_params_passed():
     Polly.auth(testpolly_token, env="testpolly")
     omix_obj = omixatlas.OmixAtlas()
-    res = omix_obj.update(repo_key, description=description)
+    # ingestion_test_1 repo
+    repo_id = "1654268055800"
+    with pytest.raises(
+        paramException,
+        match=r".* parameters passed to update, please pass at least one of the following .*",
+    ):
+        omix_obj.update(repo_key=repo_id)
+
+
+# Test 2 -> wrong description type
+def test_update_oa_wrong_description_type():
+    Polly.auth(testpolly_token, env="testpolly")
+    omix_obj = omixatlas.OmixAtlas()
+    # ingestion_test_1 repo
+    repo_id = "1654268055800"
+    with pytest.raises(
+        paramException,
+        match=r".* should be a string.*",
+    ):
+        omix_obj.update(
+            repo_key=repo_id, description=["Created Omixatlas to test description"]
+        )
+
+
+# Test 3 -> wrong display name type
+def test_update_oa_wrong_display_name_type():
+    Polly.auth(testpolly_token, env="testpolly")
+    omix_obj = omixatlas.OmixAtlas()
+    # ingestion_test_1 repo
+    repo_id = "1654268055800"
+    with pytest.raises(
+        paramException,
+        match=r".* should be a string.*",
+    ):
+        omix_obj.update(repo_key=repo_id, display_name=1234)
+
+
+@pytest.fixture()
+def update_default_patch_req_fixture():
+    # put the response in a json
+    # and do json.loads to load it into a dictionary
+    base_polly_py_test_url = test_const.POLLY_PY_TEST_FILES_URL
+    parent_dir = os.getcwd()
+    test_dir = "tests"
+    test_path = os.path.join(parent_dir, test_dir)
+    if not os.path.isdir(test_path):
+        os.makedirs(test_path)
+
+    create_response_file = f"{base_polly_py_test_url}/update_pos_response.json"
+    create_post_response_json = requests.get(create_response_file)
+
+    error_handler(create_post_response_json)
+    fake_create_post_response_json_dict = json.loads(create_post_response_json.text)
+    return fake_create_post_response_json_dict
+
+
+# Test 4 -> positive case -> update description
+def test_update_function_by_updating_description(
+    mocker, update_default_patch_req_fixture
+):
+    # updated description value
+    description_val = "updated description of lib repo dev v1"
+    # ingestion_test_1
+    repo_key_val = "1654268055800"
+    Polly.auth(testpolly_token, env="testpolly")
+    nobj = omixatlas.OmixAtlas()
+    # mocked response with status and no specific value
+    # as no specific response is mocked -> generic message will be printed
+    response = mocker.patch.object(nobj.session, "patch")
+    # http create response code -> 201
+    response.return_value.status_code = 200
+    response.return_value.json.return_value = update_default_patch_req_fixture
+
+    res = nobj.update(repo_key=repo_key_val, description=description_val)
+
     description_value = res.iloc[0, 4]
-    assert description_value == "updated description ingestion_test_1 omixatlas"'''
+    assert description_value == description_val
 
 
+# create omixatlas tests
 def test_create_oa_wrong_category_value():
     # Passing wrong value of category -> will throw error
     Polly.auth(testpolly_token, env="testpolly")
@@ -662,6 +857,60 @@ def test_create_oa_wrong_image_url_type():
         )
 
 
+@pytest.fixture()
+def create_default_param_res_fixture_df():
+    Polly.auth(testpolly_token, env="testpolly")
+    res_dict = {}
+    res_dict["Repository Id"] = "1684738195734"
+    res_dict["Repository Name"] = "doing_test_for_mocking_v5"
+    res_dict["Category"] = "private"
+    res_dict["Display Name"] = "doing_test_for_mocking_v5"
+    res_dict["Description"] = "testing locally create for mocking v5"
+    df = pd.DataFrame([res_dict])
+    return df
+
+
+@pytest.fixture()
+def create_default_post_req_fixture():
+    # put the response in a json
+    # and do json.loads to load it into a dictionary
+    base_polly_py_test_url = test_const.POLLY_PY_TEST_FILES_URL
+    parent_dir = os.getcwd()
+    test_dir = "tests"
+    test_path = os.path.join(parent_dir, test_dir)
+    if not os.path.isdir(test_path):
+        os.makedirs(test_path)
+
+    create_response_file = f"{base_polly_py_test_url}/create_pos_response.json"
+    create_post_response_json = requests.get(create_response_file)
+
+    error_handler(create_post_response_json)
+    fake_create_post_response_json_dict = json.loads(create_post_response_json.text)
+    return fake_create_post_response_json_dict
+
+
+def test_create_default_params(
+    mocker, create_default_param_res_fixture_df, create_default_post_req_fixture
+):
+    # then mocking the response of PATCH API CALL for Update Schema
+    Polly.auth(testpolly_token, env="testpolly")
+    nobj = omixatlas.OmixAtlas()
+    # mocked response with status and no specific value
+    # as no specific response is mocked -> generic message will be printed
+    response = mocker.patch.object(nobj.session, "post")
+    # http create response code -> 201
+    response.return_value.status_code = 201
+    response.return_value.json.return_value = create_default_post_req_fixture
+
+    # create default params
+    display_name = "doing_test_for_mocking_v5"
+    description = "testing locally create for mocking v5"
+
+    res = nobj.create(display_name, description)
+
+    pd.testing.assert_frame_equal(res, create_default_param_res_fixture_df)
+
+
 def test_validate_schema_function_empty_repo_id():
     # load data
     response = requests.get(const.SCHEMA_VALIDATION.get("empty_repo_id"))
@@ -1330,52 +1579,53 @@ def test_update_dataset_dict_type_source_folder_path_wrong_data_path():
 #     num_of_rows = len(res.index)
 #     assert num_of_rows == 2
 
+# TODO: need to revert commented code
+# def test_update_dataset_update_metadata_with_no_dataset_in_oa():
+#     """
+#     updating a metadata with no data in the oa
+#     shall throw a warning and skip the update
+#     """
+#     Polly.auth(testpolly_token, env="testpolly")
+#     omix_obj = omixatlas.OmixAtlas()
+#     # ingestion_test_1
+#     repo_id = "1654268055800"
+#     base_add_metadata_test_file_path = BASE_TEST_FORMAT_CONSTANTS_URL
 
-def test_update_dataset_update_metadata_with_no_dataset_in_oa():
-    """
-    updating a metadata with no data in the oa
-    shall throw a warning and skip the update
-    """
-    Polly.auth(testpolly_token, env="testpolly")
-    omix_obj = omixatlas.OmixAtlas()
-    # ingestion_test_1
-    repo_id = "1654268055800"
-    base_add_metadata_test_file_path = BASE_TEST_FORMAT_CONSTANTS_URL
+#     # creating directory
+#     parent_dir = os.getcwd()
+#     metadata_dir = "metadata_name_new"
+#     metadata_path = os.path.join(parent_dir, metadata_dir)
 
-    # creating directory
-    parent_dir = os.getcwd()
-    metadata_dir = "metadata_name_new"
-    metadata_path = os.path.join(parent_dir, metadata_dir)
+#     # metadata directory
+#     if not os.path.isdir(metadata_path):
+#         os.makedirs(metadata_path)
+#     metadata_file_folder_path = (
+#         f"{base_add_metadata_test_file_path}/metadata_name_ext_checks"
+#     )
+#     # metadata file such that corresponding data file is not present in OA
+#     metadata_file_1 = (
+#         f"{metadata_file_folder_path}/"
+#         + "DLS_BB057_CV244_AML_BMMC_ATAC088_ATAC089_GEX062_GEX063_MTDNA073_MTDNA074_0.05rnaclustres_0.05corr.json"
+#     )
+#     metadata_resp_1 = requests.get(metadata_file_1)
+#     error_handler(metadata_resp_1)
 
-    # metadata directory
-    if not os.path.isdir(metadata_path):
-        os.makedirs(metadata_path)
-    metadata_file_folder_path = (
-        f"{base_add_metadata_test_file_path}/metadata_name_ext_checks"
-    )
-    # metadata file such that corresponding data file is not present in OA
-    metadata_file_1 = (
-        f"{metadata_file_folder_path}/"
-        + "DLS_BB057_CV244_AML_BMMC_ATAC088_ATAC089_GEX062_GEX063_MTDNA073_MTDNA074_0.05rnaclustres_0.05corr.json"
-    )
-    metadata_resp_1 = requests.get(metadata_file_1)
-    error_handler(metadata_resp_1)
+#     meta_file_1_nam = "DLS_BB057_CV244_AML_BMMC_ATAC088_ATAC089_GEX062_GEX063_
+#     MTDNA073_MTDNA074_0.05rnaclustres_0.05corrN.json"
 
-    meta_file_1_nam = "DLS_BB057_CV244_AML_BMMC_ATAC088_ATAC089_GEX062_GEX063_MTDNA073_MTDNA074_0.05rnaclustres_0.05corrN.json"
+#     # creating files in metadata path
+#     with open(os.path.join(metadata_path, meta_file_1_nam), "w") as m_file_1:
+#         metadata_1_content = metadata_resp_1.text
+#         m_file_1.write(metadata_1_content)
+#     source_folder_path = {"metadata": metadata_path}
+#     destination_folder = "transcriptomics_75"
 
-    # creating files in metadata path
-    with open(os.path.join(metadata_path, meta_file_1_nam), "w") as m_file_1:
-        metadata_1_content = metadata_resp_1.text
-        m_file_1.write(metadata_1_content)
-    source_folder_path = {"metadata": metadata_path}
-    destination_folder = "transcriptomics_75"
-
-    with pytest.warns(Warning) as record:
-        omix_obj.update_datasets(repo_id, source_folder_path, destination_folder)
-        if not record:
-            pytest.fail("Expected a warning!")
-    print(record)
-    assert len(record) >= 1
+#     with pytest.warns(Warning) as record:
+#         omix_obj.update_datasets(repo_id, source_folder_path, destination_folder)
+#         if not record:
+#             pytest.fail("Expected a warning!")
+#     print(record)
+#     assert len(record) >= 1
 
 
 # these files are there in s3 but not ingested in infra
@@ -2477,7 +2727,7 @@ def test_download_dataset(mocker):
 
     with pytest.raises(
         paramException,
-        match=r"paramException \(parameter error\): dataset_ids is required and should be a list of dataset_ids as strings*",
+        match=r"paramException \(parameter error\): dataset_ids should be list of strings*",
     ):
         obj.download_dataset(repo_key, invalid_dataset_id_type, folder_path)
 
